Index: org/jivesoftware/smack/PacketWriter.java
===================================================================
--- org/jivesoftware/smack/PacketWriter.java	(révision 10999)
+++ org/jivesoftware/smack/PacketWriter.java	(copie de travail)
@@ -20,16 +20,16 @@
 
 package org.jivesoftware.smack;
 
-import org.jivesoftware.smack.filter.PacketFilter;
-import org.jivesoftware.smack.packet.Packet;
-
 import java.io.IOException;
 import java.io.Writer;
 import java.util.Map;
-import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ArrayBlockingQueue;
 import java.util.concurrent.BlockingQueue;
+import java.util.concurrent.ConcurrentHashMap;
 
+import org.jivesoftware.smack.filter.PacketFilter;
+import org.jivesoftware.smack.packet.Packet;
+
 /**
  * Writes packets to a XMPP server. Packets are sent using a dedicated thread. Packet
  * interceptors can be registered to dynamically modify packets before they're actually
@@ -299,9 +299,11 @@
             }
         }
         catch (IOException ioe){
-            if (!done) {
-                done = true;
-                connection.packetReader.notifyConnectionError(ioe);
+            synchronized (writer) {
+                if (!done) {
+                    done = true;
+                    connection.packetReader.notifyConnectionError(ioe);
+                }
             }
         }
     }
@@ -434,21 +436,27 @@
                 // Do nothing
             }
             while (!done && keepAliveThread == thread) {
+                long timeToWait = delay - (System.currentTimeMillis() - lastActive);
                 synchronized (writer) {
                     // Send heartbeat if no packet has been sent to the server for a given time
-                    if (System.currentTimeMillis() - lastActive >= delay) {
+                    if (timeToWait <= 0) {
                         try {
                             writer.write(" ");
                             writer.flush();
                         }
                         catch (Exception e) {
-                            // Do nothing
+                            if (!done) {
+                                done = true;
+                                connection.packetReader.notifyConnectionError(e);
+                            }
                         }
+                        lastActive = System.currentTimeMillis();
+                        timeToWait = delay;
                     }
                 }
                 try {
                     // Sleep until we should write the next keep-alive.
-                    Thread.sleep(delay);
+                    Thread.sleep(timeToWait);
                 }
                 catch (InterruptedException ie) {
                     // Do nothing
